@@ -55,6 +55,7 @@ import net.sourceforge.jnlp.config.DeploymentConfiguration;
 import net.sourceforge.jnlp.runtime.Translator;
 import net.sourceforge.jnlp.security.KeyStores;
 import net.sourceforge.jnlp.security.viewer.CertificatePane;
+import net.sourceforge.jnlp.util.ImageResources;
 
 /**
  * This is the control panel for Java. It provides a GUI for modifying the
@@ -102,6 +103,7 @@ public class ControlPanel extends JFrame {
     public ControlPanel(DeploymentConfiguration config) {
         super();
         setTitle(Translator.R("CPHead"));
+        setIconImages(ImageResources.INSTANCE.getApplicationImages());
 
         this.config = config;
 
@@ -115,7 +117,6 @@ public class ControlPanel extends JFrame {
         setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
         pack();
         setMinimumSize(getPreferredSize());
-        setResizable(false);
     }
 
     private JPanel createTopPanel() {
@@ -232,9 +233,23 @@ public class ControlPanel extends JFrame {
 
         // Add panels.
         final JPanel settingsPanel = new JPanel(new CardLayout());
+
+        // Calculate largest minimum size we should use.
+        int height = 0;
+        int width = 0;
         for (SettingsPanel panel : panels) {
             JPanel p = panel.getPanel();
-            p.setPreferredSize(new Dimension(530, 360));
+            Dimension d = p.getMinimumSize();
+            if (d.height > height)
+                height = d.height;
+            if (d.width > width)
+                width = d.width;
+        }
+        Dimension dim = new Dimension(width, height);
+
+        for (SettingsPanel panel : panels) {
+            JPanel p = panel.getPanel();
+            p.setPreferredSize(dim);
             settingsPanel.add(p, panel.toString());
         }
 
