@@ -16,60 +16,38 @@
 
 package net.sourceforge.jnlp.runtime;
 
-import java.awt.EventQueue;
-import java.awt.Image;
-import java.io.File;
-import java.io.FileInputStream;
-import java.io.FileOutputStream;
-import java.io.IOException;
-import java.io.PrintStream;
-import java.lang.reflect.Constructor;
-import java.lang.reflect.Method;
-import java.lang.reflect.InvocationTargetException;
+import java.io.*;
 import java.net.Authenticator;
 import java.net.ProxySelector;
 import java.nio.channels.FileChannel;
 import java.nio.channels.FileLock;
-import java.security.AllPermission;
-import java.security.KeyStore;
-import java.security.Policy;
-import java.security.Security;
-import java.security.cert.CertificateException;
-import java.security.cert.X509Certificate;
-import java.text.MessageFormat;
+import java.awt.*;
+import java.text.*;
+import java.util.*;
 import java.util.List;
-import java.util.ResourceBundle;
-
-import javax.jnlp.ServiceManager;
+import java.security.*;
+import javax.jnlp.*;
 import javax.naming.ConfigurationException;
 import javax.net.ssl.HttpsURLConnection;
 import javax.net.ssl.KeyManagerFactory;
 import javax.net.ssl.SSLContext;
-import javax.net.ssl.SSLEngine;
-import javax.net.ssl.SSLSocket;
 import javax.net.ssl.SSLSocketFactory;
 import javax.net.ssl.TrustManager;
 import javax.swing.UIManager;
 import javax.swing.text.html.parser.ParserDelegator;
 
-import net.sourceforge.jnlp.DefaultLaunchHandler;
-import net.sourceforge.jnlp.GuiLaunchHandler;
-import net.sourceforge.jnlp.LaunchHandler;
-import net.sourceforge.jnlp.Launcher;
+import sun.net.www.protocol.jar.URLJarFile;
+
+import net.sourceforge.jnlp.*;
 import net.sourceforge.jnlp.browser.BrowserAwareProxySelector;
-import net.sourceforge.jnlp.cache.CacheUtil;
-import net.sourceforge.jnlp.cache.DefaultDownloadIndicator;
-import net.sourceforge.jnlp.cache.DownloadIndicator;
-import net.sourceforge.jnlp.cache.UpdatePolicy;
+import net.sourceforge.jnlp.cache.*;
 import net.sourceforge.jnlp.config.DeploymentConfiguration;
 import net.sourceforge.jnlp.security.JNLPAuthenticator;
 import net.sourceforge.jnlp.security.KeyStores;
 import net.sourceforge.jnlp.security.SecurityDialogMessageHandler;
 import net.sourceforge.jnlp.security.VariableX509TrustManager;
-import net.sourceforge.jnlp.services.XServiceManagerStub;
-import net.sourceforge.jnlp.util.FileUtils;
-import net.sourceforge.jnlp.util.TeeOutputStream;
-import sun.net.www.protocol.jar.URLJarFile;
+import net.sourceforge.jnlp.services.*;
+import net.sourceforge.jnlp.util.*;
 
 /**
  * Configure and access the runtime environment.  This class
@@ -88,7 +66,7 @@ import sun.net.www.protocol.jar.URLJarFile;
  * @version $Revision: 1.19 $
  */
 public class JNLPRuntime {
-    
+
     static {
         loadResources();
     }
@@ -116,9 +94,6 @@ public class JNLPRuntime {
     /** update policy that controls when to check for updates */
     private static UpdatePolicy updatePolicy = UpdatePolicy.ALWAYS;
 
-    /** netx window icon */
-    private static Image windowIcon = null;
-
     /** whether initialized */
     private static boolean initialized = false;
 
@@ -146,16 +121,8 @@ public class JNLPRuntime {
     /** set to false to indicate another JVM should not be spawned, even if necessary */
     private static boolean forksAllowed = true;
 
-    /** all security dialogs will be consumed and pretented as being verified by user and allowed.*/
+    /** all security dialogs will be consumed and pretented as beeing verified by user and allowed.*/
     private static boolean trustAll=false;
-    /** 
-     * Header is not checked and so eg. gifar exploit is possible
-     * @see http://en.wikipedia.org/wiki/Gifar for this kind of attack.
-     * However if jar file is a bit corrupted, then it sometimes can work so 
-     * this switch can disable the header check.
-     * 
-     */
-    private static boolean ignoreHeaders=false;
 
     /** contains the arguments passed to the jnlp runtime */
     private static List<String> initialArguments;
@@ -218,17 +185,14 @@ public class JNLPRuntime {
         if (headless == false)
             checkHeadless();
 
-        if (!headless && windowIcon == null)
-            loadWindowIcon();
-
         if (!headless && indicator == null)
             indicator = new DefaultDownloadIndicator();
 
         if (handler == null) {
             if (headless) {
-                handler = new DefaultLaunchHandler();
+                handler = new DefaultLaunchHandler(System.err);
             } else {
-                handler = new GuiLaunchHandler();
+                handler = new GuiLaunchHandler(System.err);
             }
         }
 
@@ -259,7 +223,7 @@ public class JNLPRuntime {
             KeyStore ks = KeyStores.getKeyStore(KeyStores.Level.USER, KeyStores.Type.CLIENT_CERTS);
             KeyManagerFactory kmf = KeyManagerFactory.getInstance("SunX509");
             kmf.init(ks, KeyStores.getPassword());
-            TrustManager[] trust = new TrustManager[] { getSSLSocketTrustManager() };
+            TrustManager[] trust = new TrustManager[] { VariableX509TrustManager.getInstance() };
             context.init(kmf.getKeyManagers(), trust, null);
             sslSocketFactory = context.getSocketFactory();
 
@@ -283,52 +247,6 @@ public class JNLPRuntime {
 
     }
 
-    /**
-     * Returns a TrustManager ideal for the running VM.
-     *
-     * @return TrustManager the trust manager to use for verifying https certificates
-     */
-    private static TrustManager getSSLSocketTrustManager() throws
-                                ClassNotFoundException, IllegalAccessException, InstantiationException, InvocationTargetException {
-
-        try {
-
-            Class<?> trustManagerClass;
-            Constructor<?> tmCtor = null;
-
-            if (System.getProperty("java.version").startsWith("1.6")) { // Java 6
-                try {
-                    trustManagerClass = Class.forName("net.sourceforge.jnlp.security.VariableX509TrustManagerJDK6");
-                 } catch (ClassNotFoundException cnfe) {
-                     System.err.println("Unable to find class net.sourceforge.jnlp.security.VariableX509TrustManagerJDK6");
-                     return null;
-                 }
-            } else { // Java 7 or more (technically could be <= 1.5 but <= 1.5 is unsupported)
-                try {
-                    trustManagerClass = Class.forName("net.sourceforge.jnlp.security.VariableX509TrustManagerJDK7");
-                 } catch (ClassNotFoundException cnfe) {
-                     System.err.println("Unable to find class net.sourceforge.jnlp.security.VariableX509TrustManagerJDK7");
-                     return null;
-                 }
-            }
-
-            Constructor<?>[] tmCtors = trustManagerClass.getDeclaredConstructors();
-            tmCtor = tmCtors[0];
-
-            for (Constructor<?> ctor : tmCtors) {
-                if (tmCtor.getGenericParameterTypes().length == 0) {
-                    tmCtor = ctor;
-                    break;
-                }
-            }
-
-            return (TrustManager) tmCtor.newInstance();
-        } catch (RuntimeException e) {
-            System.err.println("Unable to load JDK-specific TrustManager. Was this version of IcedTea-Web compiled with JDK6?");
-            throw e;
-        }
-    }
-
     /**
      * This must NOT be called form the application ThreadGroup. An application
      * can inject events into its {@link EventQueue} and bypass the security
@@ -411,24 +329,6 @@ public class JNLPRuntime {
         return isWebstartApplication;
     }
 
-    /**
-     * Returns the window icon.
-     */
-    public static Image getWindowIcon() {
-        return windowIcon;
-    }
-
-    /**
-     * Sets the window icon that is displayed in Java applications
-     * and applets instead of the default Java icon.
-     *
-     * @throws IllegalStateException if caller is not the exit class
-     */
-    public static void setWindowIcon(Image image) {
-        checkExitClass();
-        windowIcon = image;
-    }
-
     /**
      * Returns whether the JNLP client will use any AWT/Swing
      * components.
@@ -697,22 +597,6 @@ public class JNLPRuntime {
         }
     }
 
-    /**
-     * Load the window icon.
-     */
-    private static void loadWindowIcon() {
-        if (windowIcon != null)
-            return;
-
-        try {
-            windowIcon = new javax.swing.ImageIcon((new sun.misc.Launcher())
-                        .getClassLoader().getResource("net/sourceforge/jnlp/resources/netx-icon.png")).getImage();
-        } catch (Exception ex) {
-            if (JNLPRuntime.isDebug())
-                ex.printStackTrace();
-        }
-    }
-
     /**
      * @return true if running on Windows
      */
@@ -824,14 +708,4 @@ public class JNLPRuntime {
         return trustAll;
     }
 
-    public static boolean isIgnoreHeaders() {
-        return ignoreHeaders;
-    }
-
-    public static void setIgnoreHeaders(boolean ignoreHeaders) {
-        JNLPRuntime.ignoreHeaders = ignoreHeaders;
-    }
-    
-    
-
 }
