@@ -180,12 +180,12 @@ public class SecurityDialogs {
      *
      * @param accessType the type of warning dialog to show
      * @param file the JNLPFile associated with this warning
-     * @param jarSigner the JarSigner used to verify this application
+     * @param certVerifier the JarCertVerifier used to verify this application
      *
      * @return true if the user accepted the certificate
      */
     public static boolean showCertWarningDialog(AccessType accessType,
-            JNLPFile file, CertVerifier jarSigner) {
+            JNLPFile file, CertVerifier certVerifier) {
 
         if (!shouldPromptUser()) {
             return false;
@@ -195,7 +195,7 @@ public class SecurityDialogs {
         message.dialogType = DialogType.CERT_WARNING;
         message.accessType = accessType;
         message.file = file;
-        message.certVerifier = jarSigner;
+        message.certVerifier = certVerifier;
 
         Object selectedValue = getUserResponse(message);
 
